@@ -20,6 +20,7 @@ from werkzeug.routing import Rule
 
 from flask_limiter import Limiter
 from flask_limiter.constants import ConfigVars, ExemptionScope, HeaderNames
+from flask_limiter.util import get_qualified_name
 from flask_limiter.wrappers import Limit
 
 limiter_theme = Theme(
@@ -68,7 +69,7 @@ def render_strategy(strategy: RateLimiter) -> str:
 def render_limit_state(
     limiter: Limiter, endpoint: str, limit: Limit, key: str, method: str
 ) -> str:
-    args = limit.args_for(endpoint, key, method)
+    args = [key, limit.scope_for(endpoint, method)]
     if not limiter.storage or (limiter.storage and not limiter.storage.check()):
         return ": [error]Storage not available[/error]"
     test = limiter.limiter.test(limit.limit, *args)
@@ -117,12 +118,16 @@ def render_limits(
 
     for limit in limits:
         if endpoint:
+            view_func = app.view_functions.get(endpoint, None)
             source = (
                 "blueprint"
                 if blueprint
                 and limit in limiter.limit_manager.blueprint_limits(app, blueprint)
                 else "route"
-                if limit in limiter.limit_manager.route_limits(app, endpoint)
+                if limit
+                in limiter.limit_manager.decorated_limits(
+                    get_qualified_name(view_func) if view_func else ""
+                )
                 else "default"
             )
         else:
@@ -525,7 +530,8 @@ def clear(
                     for limit in application_limits:
                         limiter.limiter.clear(
                             limit.limit,
-                            *limit.args_for("", key, method),
+                            key,
+                            limit.scope_for("", method),
                         )
                         node.add(f"{render_limit(limit)}: [success]Cleared[/success]")
                     console.print(node)
@@ -542,12 +548,14 @@ def clear(
                                 for rule_method in details["rule"].methods:
                                     limiter.limiter.clear(
                                         limit.limit,
-                                        *limit.args_for(endpoint, key, rule_method),
+                                        key,
+                                        limit.scope_for(endpoint, rule_method),
                                     )
                             else:
                                 limiter.limiter.clear(
                                     limit.limit,
-                                    *limit.args_for(endpoint, key, method),
+                                    key,
+                                    limit.scope_for(endpoint, method),
                                 )
                             node.add(
                                 f"{render_limit(limit)}: [success]Cleared[/success]"
