@@ -70,9 +70,9 @@ class Limiter:
     """
     The :class:`Limiter` class initializes the Flask-Limiter extension.
 
-    :param app: :class:`flask.Flask` instance to initialize the extension with.
     :param key_func: a callable that returns the domain to rate limit
       by.
+    :param app: :class:`flask.Flask` instance to initialize the extension with.
     :param default_limits: a variable list of strings or callables
      returning strings denoting default limits to apply to all routes.
      :ref:`ratelimit-string` for  more details.
@@ -126,8 +126,9 @@ class Limiter:
 
     def __init__(
         self,
+        key_func: Callable[[], str],
+        *,
         app: Optional[flask.Flask] = None,
-        key_func: Optional[Callable[[], str]] = None,
         default_limits: Optional[List[Union[str, Callable[[], str]]]] = None,
         default_limits_per_method: Optional[bool] = None,
         default_limits_exempt_when: Optional[Callable[[], bool]] = None,
@@ -186,8 +187,6 @@ class Limiter:
         self._swallow_errors = swallow_errors
         self._fail_on_first_breach = fail_on_first_breach
         self._on_breach = on_breach
-        # No longer optional
-        assert key_func
 
         self._key_func = key_func
         self._key_prefix = key_prefix
@@ -210,6 +209,7 @@ class Limiter:
                     limit_provider=limit,
                     key_function=self._key_func,
                     scope="global",
+                    shared=True,
                 )
                 for limit in application_limits
             ]
@@ -240,10 +240,8 @@ class Limiter:
         self.limit_manager = LimitManager(
             application_limits=_application_limits,
             default_limits=_default_limits,
-            static_decorated_limits={},
-            dynamic_decorated_limits={},
-            static_blueprint_limits={},
-            dynamic_blueprint_limits={},
+            decorated_limits={},
+            blueprint_limits={},
             route_exemptions=self._route_exemptions,
             blueprint_exemptions=self._blueprint_exemptions,
         )
@@ -289,9 +287,7 @@ class Limiter:
             self._headers_enabled = bool(config.get(ConfigVars.HEADERS_ENABLED, False))
 
         self._storage_options.update(config.get(ConfigVars.STORAGE_OPTIONS, {}))
-        storage_uri_from_config = config.get(
-            ConfigVars.STORAGE_URI, config.get(ConfigVars.STORAGE_URL, None)
-        )
+        storage_uri_from_config = config.get(ConfigVars.STORAGE_URI, None)
         if not storage_uri_from_config:
             if not self._storage_uri:
                 warnings.warn(
@@ -355,6 +351,7 @@ class Limiter:
                         limit_provider=app_limits,
                         key_function=self._key_func,
                         scope="global",
+                        shared=True,
                         cost=self._application_limits_cost,
                     )
                 ]
@@ -390,10 +387,6 @@ class Limiter:
             self.limit_manager.set_default_limits(default_limit_groups)
         self.__configure_fallbacks(app, self._strategy)
 
-        # purely for backward compatibility as stated in flask documentation
-        if not hasattr(app, "extensions"):
-            app.extensions = {}  # pragma: no cover
-
         if self not in app.extensions.setdefault("limiter", set()):
             if self._auto_check:
                 app.before_request(self._check_request_limit)
@@ -416,6 +409,7 @@ class Limiter:
     def limit(
         self,
         limit_value: Union[str, Callable[[], str]],
+        *,
         key_func: Optional[Callable[[], str]] = None,
         per_method: bool = False,
         methods: Optional[List[str]] = None,
@@ -427,6 +421,7 @@ class Limiter:
             Callable[[RequestLimit], Optional[flask.wrappers.Response]]
         ] = None,
         cost: Union[int, Callable[[], int]] = 1,
+        scope: Optional[Union[str, Callable[[str], str]]] = None,
     ) -> LimitDecorator:
         """
         Decorator to be used for rate limiting individual routes or blueprints.
@@ -460,6 +455,10 @@ class Limiter:
          raised.
         :param cost: The cost of a hit or a function that
          takes no parameters and returns the cost as an integer (Default: ``1``).
+        :param scope: a string or callable that returns a string
+         for further categorizing the rate limiting scope. This scope is combined
+         with the current endpoint of the request.
+
 
         Changes
           - .. versionadded:: 2.9.0 The returned object can also be used as a context manager
@@ -477,6 +476,8 @@ class Limiter:
             self,
             limit_value,
             key_func,
+            False,
+            scope,
             per_method=per_method,
             methods=methods,
             error_message=error_message,
@@ -491,6 +492,7 @@ class Limiter:
         self,
         limit_value: Union[str, Callable[[], str]],
         scope: Union[str, Callable[[str], str]],
+        *,
         key_func: Optional[Callable[[], str]] = None,
         per_method: bool = False,
         methods: Optional[List[str]] = None,
@@ -924,13 +926,10 @@ class Limiter:
         limit_for_header: Optional[RequestLimit] = None
         view_limits: List[RequestLimit] = []
         for lim in sorted(limits, key=lambda x: x.limit):
-            limit_scope = lim.scope or endpoint
-
             if lim.is_exempt or lim.method_exempt:
                 continue
 
-            if lim.per_method:
-                limit_scope += f":{flask.request.method}"
+            limit_scope = lim.scope_for(endpoint, flask.request.method)
             limit_key = lim.key_func()
             args = [limit_key, limit_scope]
             kwargs = {}
@@ -1060,7 +1059,7 @@ class LimitDecorator:
         self.limiter: weakref.ProxyType[Limiter] = weakref.proxy(limiter)
         self.limit_value = limit_value
         self.key_func = key_func or self.limiter._key_func
-        self.scope = scope if shared else None
+        self.scope = scope
         self.per_method = per_method
         self.methods = tuple(methods) if methods else None
         self.error_message = error_message
@@ -1070,9 +1069,10 @@ class LimitDecorator:
         self.on_breach = on_breach
         self.cost = cost
         self.is_static = not callable(self.limit_value)
+        self.shared = shared
 
     @property
-    def dynamic_limit(self) -> Optional[LimitGroup]:
+    def limit_group(self) -> LimitGroup:
         return LimitGroup(
             limit_provider=self.limit_value,
             key_function=self.key_func,
@@ -1085,24 +1085,7 @@ class LimitDecorator:
             deduct_when=self.deduct_when,
             on_breach=self.on_breach,
             cost=self.cost,
-        )
-
-    @property
-    def static_limits(self) -> List[Limit]:
-        return list(
-            LimitGroup(
-                limit_provider=self.limit_value,
-                key_function=self.key_func,
-                scope=self.scope,
-                per_method=self.per_method,
-                methods=self.methods,
-                error_message=self.error_message,
-                exempt_when=self.exempt_when,
-                override_defaults=self.override_defaults,
-                deduct_when=self.deduct_when,
-                on_breach=self.on_breach,
-                cost=self.cost,
-            )
+            shared=self.shared,
         )
 
     def __enter__(self) -> None:
@@ -1112,14 +1095,9 @@ class LimitDecorator:
         # TODO: if use as a context manager becomes interesting/valuable
         #  a less hacky approach than using the traceback and piggy backing
         #  on the limit manager's knowledge of decorated limits might be worth it.
-        if not self.is_static:
-            self.limiter.limit_manager.add_decorated_runtime_limit(
-                qualified_location, self.dynamic_limit
-            )
-        else:
-            self.limiter.limit_manager.add_decorated_static_limit(
-                qualified_location, *self.static_limits
-            )
+        self.limiter.limit_manager.add_decorated_limit(
+            qualified_location, self.limit_group, override=True
+        )
 
         self.limiter.limit_manager.add_endpoint_hint(
             flask.request.endpoint, qualified_location
@@ -1148,44 +1126,17 @@ class LimitDecorator:
     def __call__(
         self, obj: Union[Callable[P, R], flask.Blueprint]
     ) -> Optional[Callable[P, R]]:
-        is_route = not isinstance(obj, flask.Blueprint)
         if isinstance(obj, flask.Blueprint):
             name = obj.name
         else:
             name = get_qualified_name(obj)
-        dynamic_limit = self.dynamic_limit if not self.is_static else None
-        static_limits = []
-        if self.is_static:
-            try:
-                static_limits = self.static_limits
-            except ValueError as e:
-                self.limiter.logger.error(
-                    "failed to configure %s %s (%s)",
-                    "view function" if is_route else "blueprint",
-                    name,
-                    e,
-                )
 
         if isinstance(obj, flask.Blueprint):
-            if not self.is_static:
-                self.limiter.limit_manager.add_runtime_blueprint_limits(
-                    name, dynamic_limit
-                )
-            else:
-                self.limiter.limit_manager.add_static_blueprint_limits(
-                    name, *static_limits
-                )
+            self.limiter.limit_manager.add_blueprint_limit(name, self.limit_group)
             return None
         else:
             self.limiter._marked_for_limiting.add(name)
-            if not self.is_static:
-                self.limiter.limit_manager.add_decorated_runtime_limit(
-                    name, dynamic_limit
-                )
-            else:
-                self.limiter.limit_manager.add_decorated_static_limit(
-                    name, *static_limits
-                )
+            self.limiter.limit_manager.add_decorated_limit(name, self.limit_group)
 
             @wraps(obj)
             def __inner(*a: P.args, **k: P.kwargs) -> R:
